#!/usr/bin/env python
# encoding=utf-8
from __future__ import absolute_import

import sys

import feedparser
import requests

from . import __useragent__


plugins = {}


def register_plugin(name):
    def wrap(cls):
        plugins[name] = cls
        return cls
    return wrap


class Plugin(object):
    check_duplicates = True

    def __init__(self, key, rds=None, textme=None):
        self.key = key
        self.rds = rds
        self.textme = textme

    def get_links(self):
        for link in self._get_links():
            if self.textme:
                link = self.textme(link)
            yield link


@register_plugin('redis')
class RedisPlugin(Plugin):
    check_duplicates = False

    def _get_links(self):
        if self.rds:
            while 1:
                link = self.rds.lpop(self.key)
                if not link:
                    break
                yield None, link


@register_plugin('rss')
class RSSPlugin(Plugin):

    def _get_links(self):
        for entry in feedparser.parse(self.key).entries:
            link = None
            for x in entry.links:
                if x.get('type') == 'application/x-bittorrent' and 'href' in x:
                    link = x['href']
                    break
            if not link and entry.link:
                link = entry.link
            if link:
                yield entry.title, link


@register_plugin('json')
class JSONPlugin(Plugin):

    def _get_links(self):
        data = requests.get(self.key, headers={
            'User-Agent': __useragent__,
        }).json()
        if isinstance(data, list):
            data = {
                'items': data,
            }
        for item in data.get('items', []):
            if 'title' in item:
                title = item['title']
            else:
                title = None
            yield title, item['link']


if __name__ == '__main__':
    print(plugins)
